@@ -3,6 +3,7 @@
 {-# LANGUAGE NamedFieldPuns #-}
 {-# LANGUAGE RecordWildCards #-}
 {-# LANGUAGE ScopedTypeVariables #-}
+{-# OPTIONS_GHC -Wno-name-shadowing #-}
 
 -- SPDX-FileCopyrightText: Copyright (c) 2025 Objectionary.com
 -- SPDX-License-Identifier: MIT
@@ -12,22 +13,19 @@ module CLI (runCLI) where
 import AST
 import qualified Canonizer as C
 import Condition (parseConditionThrows)
-import Control.Exception (Exception (displayException), SomeException, handle, throw, throwIO)
-import Control.Exception.Base
+import Control.Exception.Base (Exception (displayException), SomeException, catch, fromException, handle, throwIO)
 import Control.Monad (forM_, unless, when, (>=>))
 import Data.Char (toLower, toUpper)
 import Data.Foldable (for_)
-import qualified Data.Foldable
 import Data.Functor ((<&>))
 import Data.List (intercalate)
 import Data.Maybe (fromJust, isJust, isNothing)
 import Data.Version (showVersion)
 import Dataize (DataizeContext (DataizeContext), dataize)
 import Deps (SaveStepFunc, saveStep)
-import Encoding (Encoding (ASCII, UNICODE))
+import Encoding (Encoding (UNICODE))
 import qualified Filter as F
 import Functions (buildTerm)
-import qualified Functions
 import LaTeX (LatexContext (LatexContext), explainRules, programToLaTeX, rewrittensToLatex)
 import Lining (LineFormat (MULTILINE, SINGLELINE))
 import Logger
@@ -39,7 +37,7 @@ import Options.Applicative
 import Parser (parseExpressionThrows, parseProgramThrows)
 import Paths_phino (version)
 import qualified Printer as P
-import Rewriter (RewriteContext (RewriteContext), Rewritten (..), rewrite')
+import Rewriter (RewriteContext (RewriteContext), Rewritten, rewrite')
 import Rule (RuleContext (RuleContext), matchProgramWithRule)
 import Sugar
 import System.Exit (ExitCode (..), exitFailure)
@@ -62,14 +60,14 @@ data PrintProgramContext = PrintProgCtx
   }
 
 data CmdException
-  = InvalidCLIArguments {message :: String}
-  | CouldNotReadFromStdin {message :: String}
+  = InvalidCLIArguments String
+  | CouldNotReadFromStdin String
   | CouldNotDataize
   deriving (Exception)
 
 instance Show CmdException where
-  show InvalidCLIArguments{..} = printf "Invalid set of arguments: %s" message
-  show CouldNotReadFromStdin{..} = printf "Could not read input from stdin\nReason: %s" message
+  show (InvalidCLIArguments msg) = printf "Invalid set of arguments: %s" msg
+  show (CouldNotReadFromStdin msg) = printf "Could not read input from stdin\nReason: %s" msg
   show CouldNotDataize = "Could not dataize given program"
 
 data Command
@@ -98,6 +96,7 @@ data OptsDataize = OptsDataize
   , omitComments :: Bool
   , nonumber :: Bool
   , sequence :: Bool
+  , canonize :: Bool
   , depthSensitive :: Bool
   , quiet :: Bool
   , compress :: Bool
@@ -105,6 +104,7 @@ data OptsDataize = OptsDataize
   , maxCycles :: Int
   , hide :: [String]
   , show' :: [String]
+  , locator :: String
   , expression :: Maybe String
   , label :: Maybe String
   , meetPrefix :: Maybe String
@@ -251,6 +251,9 @@ optNonumber = switch (long "nonumber" <> help "Turn off equation auto numbering
 optSequence :: Parser Bool
 optSequence = switch (long "sequence" <> help "Result output contains all intermediate 𝜑-programs concatenated with EOL")
 
+optCanonize :: Parser Bool
+optCanonize = switch (long "canonize" <> help "Rename all functions attached to λ binding with F1, F2, etc.")
+
 optExpression :: Parser (Maybe String)
 optExpression = optional (strOption (long "expression" <> metavar "NAME" <> help "Name for 'phiExpression' element when rendering to LaTeX (see --output option)"))
 
@@ -337,6 +340,7 @@ dataizeParser =
             <*> optOmitComments
             <*> optNonumber
             <*> optSequence
+            <*> optCanonize
             <*> optDepthSensitive
             <*> switch (long "quiet" <> help "Don't print the result of dataization")
             <*> optCompress
@@ -344,6 +348,7 @@ dataizeParser =
             <*> optMaxCycles
             <*> optHide
             <*> optShow
+            <*> strOption (long "locator" <> metavar "FQN" <> help "Location of object to dataize. Must be a valid dispatch expression; e.g. Q.foo.bar" <> value "Q" <> showDefault)
             <*> optExpression
             <*> optLabel
             <*> optMeetPrefix
@@ -377,7 +382,7 @@ rewriteParser =
             <*> optNonumber
             <*> switch (long "in-place" <> help "Edit file in-place instead of printing to output")
             <*> optSequence
-            <*> switch (long "canonize" <> help "Rename all functions attached to λ binding with F1, F2, etc.")
+            <*> optCanonize
             <*> optCompress
             <*> optMaxDepth
             <*> optMaxCycles
@@ -464,8 +469,8 @@ runCLI args = handle handler $ do
   case cmd of
     CmdRewrite OptsRewrite{..} -> do
       validateOpts
-      excluded <- expressionsToFilter "hide" hide
-      included <- expressionsToFilter "show" show'
+      excluded <- validatedDispatches "hide" hide
+      included <- validatedDispatches "show" show'
       logDebug (printf "Amount of rewriting cycles across all the rules: %d, per rule: %d" maxCycles maxDepth)
       input <- readInput inputFile
       rules' <- getRules normalize shuffle rules
@@ -505,6 +510,8 @@ runCLI args = handle handler $ do
             logDebug (printf "The option '--in-place' is specified, writing back to '%s'..." file)
             writeFile file prog
             logDebug (printf "The file '%s' was modified in-place" file)
+          (True, _, Nothing) ->
+            error "The option --in-place requires an input file"
           (False, Just file, _) -> do
             logDebug (printf "The option '--target' is specified, printing to '%s'..." file)
             writeFile file prog
@@ -523,15 +530,17 @@ runCLI args = handle handler $ do
             (saveStepFunc stepsDir ctx)
     CmdDataize OptsDataize{..} -> do
       validateOpts
-      excluded <- expressionsToFilter "hide" hide
-      included <- expressionsToFilter "show" show'
+      excluded <- validatedDispatches "hide" hide
+      included <- validatedDispatches "show" show'
+      [loc] <- validatedDispatches "locator" [locator]
       input <- readInput inputFile
       prog <- parseProgram input inputFormat
       let printCtx = PrintProgCtx sugarType flat defaultXmirContext nonumber compress expression label meetPrefix outputFormat
+          _canonize = if canonize then C.canonize else id
           _hide = (`F.exclude` excluded)
           _show = (`F.include` included)
-      (maybeBytes, seq) <- dataize prog (context prog printCtx)
-      when sequence (printRewrittens printCtx (_hide (_show seq)) >>= putStrLn)
+      (maybeBytes, seq) <- dataize loc (context prog printCtx)
+      when sequence (printRewrittens printCtx (_canonize $ _hide $ _show seq) >>= putStrLn)
       unless quiet (putStrLn (maybe (P.printExpression ExTermination) P.printBytes maybeBytes))
       where
         validateOpts :: IO ()
@@ -556,7 +565,6 @@ runCLI args = handle handler $ do
     CmdExplain OptsExplain{..} -> do
       validateOpts
       rules' <- getRules normalize shuffle rules
-      let latex = explainRules rules'
       output targetFile (explainRules rules')
       where
         validateOpts :: IO ()
@@ -606,17 +614,17 @@ saveStepFunc stepsDir ctx@PrintProgCtx{..} = saveStep stepsDir ioToExt (printPro
       | outputFormat == LATEX = "tex"
       | otherwise = show outputFormat
 
--- Get list of expressions which must be hidden in printed programs
-expressionsToFilter :: String -> [String] -> IO [Expression]
-expressionsToFilter opt = traverse (parseExpressionThrows >=> asFilter)
+-- Validate given expressions as valid dispatches
+validatedDispatches :: String -> [String] -> IO [Expression]
+validatedDispatches opt = traverse (parseExpressionThrows >=> asDispatch)
   where
-    asFilter :: Expression -> IO Expression
-    asFilter expr = asFilter' expr
+    asDispatch :: Expression -> IO Expression
+    asDispatch expr = asDispatch' expr
       where
-        asFilter' :: Expression -> IO Expression
-        asFilter' exp@(ExDispatch ExGlobal _) = pure exp
-        asFilter' exp@(ExDispatch expr attr) = asFilter' expr >> pure exp
-        asFilter' _ =
+        asDispatch' :: Expression -> IO Expression
+        asDispatch' exp@ExGlobal = pure exp
+        asDispatch' exp@(ExDispatch ex _) = asDispatch' ex >> pure exp
+        asDispatch' _ =
           invalidCLIArguments
             ( printf
                 "Only dispatch expression started with Φ (or Q) can be used in --%s, but given: %s"
@@ -665,6 +673,7 @@ parseProgram phi PHI = parseProgramThrows phi
 parseProgram xmir XMIR = do
   doc <- parseXMIRThrows xmir
   xmirToPhi doc
+parseProgram _ LATEX = invalidCLIArguments "LaTeX cannot be used as input format"
 
 printRewrittens :: PrintProgramContext -> [Rewritten] -> IO String
 printRewrittens ctx@PrintProgCtx{..} rewrittens
