@@ -15,7 +15,7 @@ module Printer
   , printExtraArg
   , printSubsts
   , printSubsts'
-  , PrintConfig (..)
+  , PrintConfig
   , logPrintConfig
   )
 where
@@ -47,18 +47,18 @@ printProgram :: Program -> String
 printProgram prog = printProgram' prog defaultPrintConfig
 
 printExpression' :: Expression -> PrintConfig -> String
-printExpression' expr (sugar, encoding, line) = render (withLineFormat line $ withEncoding encoding $ withSugarType sugar $ expressionToCST expr)
+printExpression' ex (sugar, encoding, line) = render (withLineFormat line $ withEncoding encoding $ withSugarType sugar $ expressionToCST ex)
 
 printExpression :: Expression -> String
-printExpression expr = printExpression' expr defaultPrintConfig
+printExpression ex = printExpression' ex defaultPrintConfig
 
 printAttribute' :: Attribute -> Encoding -> String
-printAttribute' attr encoding = render (withEncoding encoding (toCST attr 0 NO_EOL :: ATTRIBUTE))
+printAttribute' att encoding = render (withEncoding encoding (toCST att 0 NO_EOL :: ATTRIBUTE))
 
 printAttribute :: Attribute -> String
-printAttribute attr =
+printAttribute att =
   let (_, encoding, _) = defaultPrintConfig
-   in printAttribute' attr encoding
+   in printAttribute' att encoding
 
 printBinding' :: Binding -> PrintConfig -> String
 printBinding' bd = printExpression' (ExFormation [bd])
@@ -70,24 +70,24 @@ printBytes :: Bytes -> String
 printBytes bts = render (toCST bts 0 NO_EOL :: BYTES)
 
 printExtraArg' :: ExtraArgument -> PrintConfig -> String
-printExtraArg' (ArgAttribute attr) (_, encoding, _) = printAttribute' attr encoding
+printExtraArg' (ArgAttribute att) (_, encoding, _) = printAttribute' att encoding
 printExtraArg' (ArgBinding bd) config = printBinding' bd config
-printExtraArg' (ArgExpression expr) config = printExpression' expr config
+printExtraArg' (ArgExpression ex) config = printExpression' ex config
 printExtraArg' (ArgBytes bts) _ = printBytes bts
 
 printExtraArg :: ExtraArgument -> String
 printExtraArg arg = printExtraArg' arg defaultPrintConfig
 
 printTail :: Tail -> PrintConfig -> String
-printTail (TaApplication tau) config = "(" <> printBinding' tau config <> ")"
-printTail (TaDispatch attr) (_, encoding, _) = "." <> printAttribute' attr encoding
+printTail (TaApplication bd) config = "(" <> printBinding' bd config <> ")"
+printTail (TaDispatch att) (_, encoding, _) = "." <> printAttribute' att encoding
 
 printMetaValue :: MetaValue -> PrintConfig -> String
-printMetaValue (MvAttribute attr) (_, encoding, _) = printAttribute' attr encoding
-printMetaValue (MvExpression expr _) config = printExpression' expr config
+printMetaValue (MvAttribute att) (_, encoding, _) = printAttribute' att encoding
+printMetaValue (MvExpression ex _) config = printExpression' ex config
 printMetaValue (MvBytes bts) _ = printBytes bts
 printMetaValue (MvBindings bds) config = printExpression' (ExFormation bds) config
-printMetaValue (MvFunction func) _ = func
+printMetaValue (MvFunction fun) _ = fun
 printMetaValue (MvTail tails) config = intercalate "," (map (`printTail` config) tails)
 
 printSubst :: Subst -> PrintConfig -> String
